# Guía de usuario MSP430 GCC Toolchain

Galván.A , Chávez.G , Julio,2022.

------

[^]: Para usuarios de Linux 

**Abstract**

This manual aims to describe the setup and basic operations of the MSP430 GCC toolchain to know how to use the open-source GCC compiler for microcontrollers to build an example for an MSP430 target device using the command terminal of Linux. It will also be shown how to debug software with the GDB Agent and the GNU Debugger tools so that the program that was built can be uploaded to the target device successfully.

[^]: This user guide was tested in the following distributions of Linux: **Ubuntu 22.04 LTS** and **Debian 11** using a **MSP430FR6989** microcontroller.
------

[TOC]

------

##  **Introducción**

El paquete GCC de código abierto es un depurador completo y una cadena de herramientas de compilación C/C++ de código abierto para la creación y depuración de aplicaciones integradas basadas en microcontroladores MSP430. Este compilador soporta todos los dispositivos MSP430 sin limitaciones de tamaño de código y se puede utilizar de forma independiente desde la línea de comandos o dentro de Code Composer Studio v6.0 o posterior [1].

*El paquete de instalación incluye lo siguiente:*

1. Binarios GCC y GDB para Windows y Linux.
2. Encabezado MSP430 y archivos de enlace.
3. Código fuente.
4. GBD Agent Configuration.

#### Instalación de MSP430 GCC como paquete independiente

El primer paso es bastante sencillo, consiste en abrir la terminal de comando y direccionarse a la carpeta en donde se guardará el archivo que contenga el programa a ejecutar.
Con el comando **cd** se puede entrar a alguna carpeta en específico como lo puede ser la carpeta de **Descargas**, Ejemplo:

```bash
$ cd Descargas
```

Antes de continuar con el siguiente paso se debe dirigir a la página oficial de TI (Texas Instruments) yendo directamente a la siguiente página [https://www.ti.com/tool/MSP430\-GCC\-OPENSOURCE](). En la pestaña de *descargas* se debe seleccionar la opción más factible para su computadora en este caso **Mitto Systems GCC 64-bit Linux installer incl. support files**. Una vez descargada, en la carpeta de descargas se debe buscar el nombre del archivo y posteriormente, en la terminal de comando se debe colocar lo siguiente: "**sudo chmod +x <installer>.run** y el comando **./<installer>**", lo cual permitirá instalar el programa. A continuación se muestra un ejemplo, donde **msp430-gcc-full-linux-X64-installer-9.3.1.2.run**, es el nombre del archivo que se descargó (installer).

```bash
$ sudo chmod +x msp430-gcc-full-linux-X64-installer-9.3.1.2.run
$ ./msp430-gcc-full-linux-X64-installer-9.3.1.2.run
```

Al ejecutar los comandos anteriormente mencionados se abrirá la siguiente ventana:

![1.png](https://i.postimg.cc/c4vD5DpJ/1.png)

Se debe seleccionar el directorio de instalación y dar click en **Next** hasta finalizar como se muestra en la siguiente imagen:

![2.png](https://i.postimg.cc/44WBMXKZ/2.png)

#### Creación de directorio

Se debe crear una carpeta en donde se colocará el código y los archivos necesarios para compilar el programa, en este caso se creó la carpeta **Programa1** como ejemplo.

> Comando para crear carpetas desde la terminal , el comando **mkdir** + el nombre del nuevo archivo.
> Comando para encontrar carpetas dentro de la carpeta en la que se encuentra **dir**.
> Comando para entrar en carpeta **cd**.

```bash
$ mkdir Programa1
$ dir 
$ cd Programa1     
```

El código que se muestra a continuación, hace parpadear el LED P1.1 del microcontrolador MSP430FR6989, este se utilizará como ejemplo a lo largo del documento. Para poder compilar y depurar el código, es necesario copiar dicho código en un archivo **.c**. 

```c
#include <msp430fr6989.h> 

void main(void)
{
	WDTCTL = WDTPW | WDTHOLD;	// stop watchdog timer
	PM5CTL0 = 0x0000;		// disable high impedance mode
	P1DIR = 0x01;   		//set up bit 0 of P1 as output
	P1OUT = 0x00;  			//initialize bit 0 of P1 to 0
	
	for(;;){    			//loop
	    volatile unsigned int i;
	    P1OUT ^= 0x01;  		//toggle bit 0 of P1
	    				//Delay
	    for(i=40000; i>0;){
	        i--;
	        __no_operation();
	    }
	}
}
```

#### Creación de un archivo .c

Para crear un archivo **.c** se debe posicionar en el directorio en el que lo desea crear y se debe digitar el comando **pico filename.c**. Por fines explicativos, como ejemplo, se creará un archivo **.c** llamado **Ejemplo** en la carpeta **Programa1** (previamente creada). Una vez que se ejecuta el comando mencionado, se abrirá una ventana en la que se deberá copiar el códiogo del programa que se quiere subir a la tarjeta. En nuestro caso se copió el código mencionado anteriormente. 

Comando para editar/crear el archivo **.c**.

[^]: Este ejemplo fue realizado con el editor de texto **pico**, pero se puede utilizar cualquier otro editor de su preferencia.

```bash
$ pico Ejemplo.c
```

Una vez copiado el código en el archivo **.c**, se teclea `Ctrl + X` para salir, posteriormente se guarda el archivo tecleando `S` y por  último se teclea `Enter` para regresar a la terminal de comandos.

#### Compilación de un programa desde la termial de Linux

El programa se puede compilar manualmente desde la terminal indicando las rutas de instalación de las librerías, binarios y demás, o usando un método más sencillo implementando un archivo *Makefile* incluido en los ejemplos por defecto que el paquete contiene. En este documento se muestran los pasos a seguir empleando el segundo método comenzando con la creación de un nuevo proyecto; sin embargo, a continuación, se muestra un ejemplo de compilación manual desde la terminal.

Ejemplo de compilación de un programa desde la terminal (primer método):

```bash
$ ./msp430-elf-gcc  -I ~/msp430-gcc/include -L ~/msp430-gcc/include -T ~/msp430-gcc/include/msp430fr6989.ld -mmcu=msp430fr6989 -O2 -g ~/Documentos/Programa1/Ejemplo.c -o ~/Documentos/Programa1/Ejemplo.o
```

Primero se debe dirigir a la dirección en donde se encuentra instalado el paquete *MSP430- GCC* y redireccionarse a la carpeta *bin*.
Para compilar se coloca ./ seguido de la dirección *msp430-elf-gcc* y después se colocan los parámetros  de las banderas (flags) correspondientes a -I, -L, -T, -g, -o, -mmcu.

Donde:

- **-I**: se agrega el directorio a la lista de directorios para buscar archivos de encabezado.
- **-L**: se agregan la ruta en las que ld buscará las bibliotecas del archivo.
- **-T**: dirección donde se encuentran las librerías del microcontrolador especificando cuales se utilizarán .
- **-g**: dirección donde se encuentra el archivo .c .
- **-o**: se especifica la ruta donde se encuentra el archivo .c, con la diferencia que se cambia el .c por .o (output file).
- **-mmcu**: especifica el microcontrolador que se utilizará.

## **Creación de un nuevo proyecto** 

------

#### Pasos para la creación de un nuevo proyecto

1. Crear un directorio para el nuevo proyecto.
2. Insertar el archivo .c que contiene el programa que se subirá a la tarjeta. Para crear un archivo .c vea el apartado **Creación de un archivo .c**.
3. Dentro de la carpeta que se instaló (msp430-gcc), hay un directorio llamado **examples** que contiene ejemplos predeterminados. Copiar uno de los archivos `Makefile` de los proyectos de ejemplo dentro del directorio de su nuevo proyecto, en este caso, dentro de la carpeta **Programa1**.
4. Abrir el archivo `Makefile` que copió y cambiar el formato de estructura actual por el que se muestra a continuación. En la variable **OBJECTS** se debe colocar el nombre del programa que se va a subir (archivo .c) respetando la terminación **.o**. En la variable **DEVICE** se debe colocar el nombre del microcontrolador que se usará.
5. Colocar las rutas de instalación correpondientes para las variables **GCC\_DIR** y **SUPPORT\_FILE\_DIRECTORY**. Las direcciones para las variables son **../../../bin** y **../../../include** respectivamente como se muestra en el siguiente ejemplo.

Estructura de archivo Makefile:

```c
OBJECTS=Ejemplo.o

GCC_DIR = /home/usuario/msp430-gcc/bin
SUPPORT_FILE_DIRECTORY = /home/usuario/msp430-gcc/include

DEVICE  = msp430fr6989
CC      = $(GCC_DIR)/msp430-elf-gcc
GDB     = $(GCC_DIR)/msp430-elf-gdb

CFLAGS = -I $(SUPPORT_FILE_DIRECTORY) -mmcu=$(DEVICE) -O2 -g
LFLAGS = -L $(SUPPORT_FILE_DIRECTORY) -T $(DEVICE).ld

all: ${OBJECTS}
	$(CC) $(CFLAGS) $(LFLAGS) $? -o $(DEVICE).out


debug: all
	$(GDB) $(DEVICE).out
```

## **Debugging**

#### **Uso del agente GDB **

###### Introducción 

El agente GDB es una herramienta para conectar el GDB con el hardware de destino para depurar el software. El GDB Agent utiliza la pila de depuración MSP430 para conectarse al hardware y proporciona una interfaz al GDB.
En Windows, se proporciona una consola y una versión de aplicación GUI del agente GDB. Solo la aplicación de consola (terminal de comandos) es compatible con Linux.

#### Iniciando GDB Agent

###### Línea de comando

Abra una terminal de comando y dirijase al directorio de instalación, una vez hecho, ejecute lo siguiente:

```bash
$ ./bin/gdb\_agent\_console msp430.dat
```

La aplicación de consola abre un puerto TCP/IP en el equipo local. Muestra el número de puerto en la consola, por defecto, este número de puerto es 55000.

[^]: Es importante que se deje abierta esta consola hasta terminar de depurar el programa (debugging) para poder conectar el GDB con el GDB Agent.

```c
$ ./bin/gdb\_agent\_console msp430.dat
CPU Name             Port
--------             ----
msp430              :55000

Starting all cores
CPU Name             Status
--------             ------
msp430               Waiting for client
```

#### Debugging con GDB

##### Ejecutar un programa en el debugger

1. Abra otra terminal de comando y dirijase a la ruta donde se encuentra el directorio con el archivo **.c** y el `Makefile` del programa que quiere cargar a la tarjeta.

2. Escriba el comando **make debug**. Este comando inicia el GDB y espera que se introduzcan comandos, esto es indicado por el puntero **(gdb)**. 

   ```bash
   $ make debug
   /home/usuario/msp430-gcc/bin/msp430-elf-gcc -I /home/usuario/msp430-gcc/include -mmcu=msp430fr6989 -O2 -g -L /home/usuario/msp430-gcc/include -T msp430fr6989.ld  Ejemplo.o -o msp430fr6989.out
   /home/usuario/msp430-gcc/bin/msp430-elf-gdb msp430fr6989.out
   GNU gdb (Mitto Systems Limited - msp430-gcc 9.3.1.11) 9.1
   Copyright (C) 2020 Free Software Foundation, Inc.
   License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
   This is free software: you are free to change and redistribute it.
   There is NO WARRANTY, to the extent permitted by law.
   Type "show copying" and "show warranty" for details.
   This GDB was configured as "--host=x86_64-pc-linux-gnu --target=msp430-elf".
   Type "show configuration" for configuration details.
   For bug reporting instructions, please see:
   <http://www.gnu.org/software/gdb/bugs/>.
   Find the GDB manual and other documentation resources online at:
       <http://www.gnu.org/software/gdb/documentation/>.
   
   For help, type "help".
   Type "apropos word" to search for commands related to "word"...
   Reading symbols from msp430fr6989.out...
   (gdb) 
   ```

3. Conecte el dispositivo.

4. Escriba el comando **target remote :55000** y presione enter para conectar el GDB con el GDB Agent que se encuentra en la otra terminal de comando que se dejó abierta. En la otra terminal se podrá observar en la consola del GDB Agent que se conectó un dispositivo/cliente.

   ```bash
   Reading symbols from msp430fr6989.out...
   (gdb) target remote :55000
   Remote debugging using :55000
   0x00004402 in __crt0_start ()
   (gdb) 
   ```

   Consola del GDB Agent:

   ```c
   $ ./bin/gdb\_agent\_console msp430.dat
   CPU Name             Port
   --------             ----
   msp430              :55000
   
   Starting all cores
   CPU Name             Status
   --------             ------
   msp430               Waiting for client
   msp430               Client connected...Connecting to Target
   Found USB FET at ttyACM0
   Target connected...Starting server
   ```

5. Escriba **load** y presione enter para cargar el programa binario al dispositivo MSP430 que esté usando.

   ```bash
   (gdb) load
   Loading section .upper.data, size 0x2 lma 0x4400
   Loading section .text, size 0x3c lma 0x4402
   Loading section __reset_vector, size 0x2 lma 0xfffe
   Start address 0x00004402, load size 64
   Transfer rate: 71 bytes/sec, 21 bytes/write.
   (gdb)
   ```

6. Escriba el comando **continue** para correr el programa o en su defecto escriba **step** para compilar el programa paso por paso.

   ```bash
   (gdb) continue
   Continuing.
   ```

7. Para parar la ejecución teclee `Ctrl + C` (^C), y para salir del GDB escriba `q`, presione enter y posteriormente escriba `y` y presione enter.

   ```bash
   ^C
   Program stopped.
   main () at Ejemplo.c:15
   15		        i--;
   (gdb) q
   A debugging session is active.
   
   	Inferior 1 [Remote target] will be detached.
   
   Quit anyway? (y or n) y
   Detaching from program: /home/usuario/Escritorio/msp430_programs/Programa1/msp430fr6989.out, Remote target
   Ending remote debugging.
   [Inferior 1 (Remote target) detached]
   ```

### **Solución de problemas**

[^]: En este apartado se muestran las posibles soluciones a los problemas más comunes.

**Error Make debug*

```bash
$ make debug
Make: *** There is no rule to build the 'debug' target. Tall
```

Instalar make para solucionar el problema mediante el comando :

```bash
sudo apt-get install make
```

------

 **Error 127 makefile debug*

```bash
/home/usuario/msp430-gcc/bin/msp430-elf-gdb:error while loading shared libraries: libncursesw.so.5: cannot open shared object file: No such file or directory 
make: *** [Makefile: debug] Error 127
```

Instalar la libreria *libncursesw.so.5* para solucionar el problema, en caso de que continúe el problema, instalar *libncursesw5-dev* mediante el comando :

```bash
sudo apt-get install libncursesw5
sudo apt-get inatall libncursesw5-dev	(opcional)
```

------

### **Referencias**

 [1] Texas Instruments.  Msp430-gcc-opensource gcc-open source compiler for msp microcontrollers, 2017.

